use mongodb::{bson::doc, ClientSession};

use crate::get_mongo;

pub struct InventoryCache {}

impl InventoryCache {
    pub async fn push(uuid: &str, ses: &mut ClientSession) {
        // todo : if not exists?
        let update = doc! { "$push": { "transactions": uuid } };

        let options = mongodb::options::FindOneAndUpdateOptions::builder()
            .upsert(true)
            .return_document(mongodb::options::ReturnDocument::After)
            .build();

        let result = ses
            .client()
            .database("cdb")
            .collection::<mongodb::bson::Document>("cache")
            .find_one_and_update_with_session(doc! { "_id": "inventory"}, update, options, ses)
            .await
            .unwrap();
    }

    pub async fn get() -> Vec<String> {
        let db = get_mongo!();
        let inventorycache = db
            .database("cdb")
            .collection::<mongodb::bson::Document>("cache")
            .find_one(doc! {"_id": "inventory"}, None)
            .await
            .unwrap()
            .unwrap();
        inventorycache
            .get_array("transactions")
            .unwrap()
            .iter()
            .map(|x| x.as_str().unwrap().to_string())
            .collect()
    }

    pub async fn remove(uuid: &str, ses: &mut ClientSession) {
        let update = doc! { "$pull": { "transactions": uuid}};

        let options = mongodb::options::FindOneAndUpdateOptions::builder()
            .upsert(true)
            .return_document(mongodb::options::ReturnDocument::After)
            .build();

        let result = ses
            .client()
            .database("cdb")
            .collection::<mongodb::bson::Document>("cache")
            .find_one_and_update_with_session(doc! { "_id": "inventory"}, update, options, ses)
            .await
            .unwrap();
    }
}
